/* ==========================================================================*/
/* - - - - - - - - - - - - - -  I r q . c  - - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver IRQ service Routines                            */
/*     (must be compiled as Non-Thum ARM mode)                               */
/*.........................................................................  */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "common.h"
#include "Board.h"
#include "fpcom.h"
#include "Usart.h"
#include "Rfcontrol.h"
#include "ambe2020.h"
#include "irq.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*---------------------> L O C A L   M E M O R Y  <--------------------------*/
/*---------------------------------------------------------------------------*/
static tDstarVDFrame TmpFrame;
static U32BIT SyncTest;

/******************************************************************************
 *
 * MACRO Name: ISR_ENTRY()
 *
 * Description:
 *    This MACRO is used upon entry to an ISR.  The current version of
 *    the gcc compiler for ARM does not produce correct code for
 *    interrupt routines to operate properly with THUMB code.  The MACRO
 *    performs the following steps:
 *
 *    1 - Adjust address at which execution should resume after servicing
 *        ISR to compensate for IRQ entry
 *    2 - Save the non-banked registers r0-r12 and lr onto the IRQ stack.
 *    3 - Get the status of the interrupted program is in SPSR.
 *    4 - Push it onto the IRQ stack as well.
 *
 *****************************************************************************/
#define ISR_ENTRY() asm volatile(" sub   lr, lr,#4\n" \
                                 " stmfd sp!,{r0-r12,lr}\n" \
                                 " mrs   r1, spsr\n" \
                                 " stmfd sp!,{r1}")

/******************************************************************************
 *
 * MACRO Name: ISR_EXIT()
 *
 * Description:
 *    This MACRO is used to exit an ISR.  The current version of the gcc
 *    compiler for ARM does not produce correct code for interrupt
 *    routines to operate properly with THUMB code.  The MACRO performs
 *    the following steps:
 *
 *    1 - Recover SPSR value from stack       
 *    2 - and restore  its value                   
 *    3 - Pop the return address & the saved general registers from
 *        the IRQ stack & return
 *
 *****************************************************************************/
#define ISR_EXIT()  asm volatile(" ldmfd sp!,{r1}\n" \
                                 " msr   spsr_c,r1\n" \
                                 " ldmfd sp!,{r0-r12,pc}^") 
  

/******************************************************************************
 *
 * MACRO Name: FSR_ENTRY()
 *
 * Description:
 *    This MACRO is used upon entry to an FSR.  The current version of
 *    the gcc compiler for ARM does not produce correct code for
 *    interrupt routines to operate properly with THUMB code.  The MACRO
 *    performs the following steps:
 *
 *    1 - Adjust address at which execution should resume after servicing
 *        FSR to compensate for FIQ entry
 *    2 - Save the non-banked registers r0-r7 and lr onto the FIQ stack.
 *
 *****************************************************************************/
#define FSR_ENTRY() asm volatile(" sub   lr, lr,#4\n" \
                                 " stmfd sp!,{r0-r7,lr}")

/******************************************************************************
 *
 * MACRO Name: FSR_EXIT()
 *
 * Description:
 *    This MACRO is used to exit an FSR.  The current version of the gcc
 *    compiler for ARM does not produce correct code for interrupt
 *    routines to operate properly with THUMB code.  The MACRO performs
 *    the following steps:
 *
 *    1 - Pop the return address & the saved general registers from
 *        the FIQ stack & return
 *
 *****************************************************************************/
#define FSR_EXIT()  asm volatile(" ldmfd sp!,{r0-r7,pc}^")


/*---------------------------------------------------------------------------*/
/*----------> I N T E R R U P T    S E R V I C E    R O U T I N E S <--------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------------                        */
/*                     | U a r t I r q S e r v i c e |                       */
/*                      -----------------------------                        */
/*    Called by uart IRQ.                                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartIrqService(void)
{
	/* This ISR can cause a context switch, so the first statement must be a
	call to the portENTER_SWITCHING_ISR() macro.  This must be BEFORE any
	variable declarations. */
	portENTER_SWITCHING_ISR();
unsigned int data;
U32BIT Status = *AT91C_US0_CSR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	if ( Status & AT91C_US_RXRDY)
	{
		if ( 0 == (Status & (AT91C_US_OVRE|AT91C_US_FRAME)) )
		{
			//* Get byte
			UsartRxQue[RxHead++] = (U8BIT)*AT91C_US0_RHR;
			RxHead &= (RXQ_SIZE-1);	//deal with rollover
			xSemaphoreGiveFromISR( xUartSemaphore, ( portBASE_TYPE )pdFALSE );
		}
		else
		{
			data = *AT91C_US0_RHR;
		}
	}
	if( Status & AT91C_US_TXRDY)
	{
		*AT91C_US0_THR = UsartTxQue[TxTail++];
		TxTail &= (TXQ_SIZE-1);	//deal with rollover
		if( TxHead == TxTail )
		{
			*AT91C_US0_IDR = AT91C_US_TXRDY;
		}
	}
	//* Reset the status bit
	*AT91C_US0_CR = AT91C_US_RSTSTA;
	/* Exit the ISR and force a context switch */
	portEXIT_SWITCHING_ISR( pdFALSE );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | S S C I r q S e r v i c e |                         */
/*                      ---------------------------                          */
/*    Called by IRQ when new data received from Sync serial interface.       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SSCIrqService(void)
{
	ISR_ENTRY();
U32BIT Status = *AT91C_SSC_SR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	if(	XFERSTATE_LOOKING_FOR_SYNC == XferState )
	{
		if( AMBE2020_FRAME_SYNC == *AT91C_SSC_RHR)
		{
			XferState = XFERSTATE_RECEIVING;
			OutFrame.All[SSCTransferIndex] = AMBE2020_FRAME_SYNC;
			*AT91C_SSC_THR = InFrame.All[SSCTransferIndex++];
		}
		else
		{
			*AT91C_SSC_THR = AMBE2020_IDLE_WORD;
		}
	}
	else if( XFERSTATE_RECEIVING == XferState )
	{
		if(SSCTransferIndex < 24)
		{
			OutFrame.All[SSCTransferIndex] = *AT91C_SSC_RHR;
			*AT91C_SSC_THR = InFrame.All[SSCTransferIndex++];
		}
		else
		{
			Status = *AT91C_SSC_RHR;
			*AT91C_SSC_TFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_HIGH;
			*AT91C_SSC_IDR = AT91C_SSC_RXRDY;
			SSCTransferIndex = 0;
			XferState = XFERSTATE_FINISHED;
			AT91C_BASE_TC2->TC_RC = TIMER2_DATARDY;
			AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer for signaling
		}
	}
	ISR_EXIT();
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------------                    */
/*                     | T i m e r 2 I r q S e r v i c e |                   */
/*                      ---------------------------------                    */
/*    Called by IRQ to service one-shot timer 2.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Timer2IrqService(void)
{
	portENTER_SWITCHING_ISR();
S32BIT tmp;
    //* Acknowledge interrupt status
    tmp = AT91C_BASE_TC2->TC_SR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	if(	XFERSTATE_FINISHED == XferState )
	{	//here if just finished getting/sending packet of ambe2020 data
		XferState = XFERSTATE_LOOKING_FOR_SYNC;
		AT91C_BASE_TC2->TC_RC = TIMER2_DELAY_TO_SYNC_SEARCH;
		AT91C_BASE_TC2->TC_CCR = AT91C_TC_SWTRG;	//trigger one shot timer just before
													//ambe2020 is ready
		xSemaphoreGiveFromISR( xAmbeBlockRdySemaphore, ( portBASE_TYPE )pdFALSE );
	}
	else
	{	//here just before ambe is ready with new frame sync so start watching process
		*AT91C_SSC_TFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_POSITIVE;
		SSCTransferIndex = 0;
		tmp = *AT91C_SSC_RHR;
		*AT91C_SSC_THR = AMBE2020_IDLE_WORD;
		*AT91C_SSC_IER = AT91C_SSC_RXRDY; 
	}
	portEXIT_SWITCHING_ISR( pdTRUE );
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------------                    */
/*                     | T i m e r 1 I r q S e r v i c e |                   */
/*                      ---------------------------------                    */
/*    GP timer.                                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Timer1IrqService(void)
{
	ISR_ENTRY();
unsigned int dummy;
AT91PS_TC TC_pt = AT91C_BASE_TC1;
    //* Acknowledge interrupt status
    dummy = TC_pt->TC_SR;
	AT91C_BASE_AIC->AIC_EOICR = 0;
	ISR_EXIT();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------------                        */
/*                     | P W M 2 I r q S e r v i c e |                       */
/*                      -----------------------------                        */
/*    Called by PWM2 to service 4800Hz Data Transmitting.                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PWM2IrqService(void)
{
	ISR_ENTRY();
S32BIT tmp;
    //* Acknowledge interrupt status
	AT91C_BASE_AIC->AIC_EOICR = 0;
    tmp = *AT91C_PWMC_ISR;
	if( TXSTATE_OFF != TxState )
	{
		if(0 == TxShiftCount)
		{	//get next 32 bit word from Xmit queue
			if( RFTxQTail != RFTxQHead)
			{
				TxShiftWord = RFTxDataQueue[RFTxQTail++];
				RFTxQTail = RFTxQTail & (RFTQ_SIZE-1);
				TxShiftCount = 32;
			}
			else
			{
				TxState = TXSTATE_DONE;
			}
		}
		if(TxShiftWord & 1 )
		{
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, TXDATA_7021 );
		}
		else
		{
			AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, TXDATA_7021 );
		}
		TxShiftWord >>= 1;
		TxShiftCount--;
	}
	ISR_EXIT();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -----------------------------                           */
/*                  | P I O A I r q S e r v i c e |                          */
/*                   -----------------------------                           */
/*  Service of PIO change interrupts.                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PIOAIrqService(void)
{
	ISR_ENTRY();
U32BIT tmp;
     //* Acknowledge interrupt status
	AT91C_BASE_AIC->AIC_EOICR = 0;
	tmp = *AT91C_PIOA_ISR;		//clears irq status bits as well
	if( (*AT91C_PIOA_PDSR&(UPSW_|DNSW_)) == 0 )  
	{	//if transition to 00 state then determine direction
		if(tmp&UPSW_)
			RotaryChange++;
		else
			RotaryChange--;
	}
	ISR_EXIT();
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------------------                   */
/*                  | S p u r i o u s I r q S e r v i c e |                  */
/*                   -------------------------------------                   */
/*  Must have do nothing handler for spurious interrupts.                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SpuriousIrqService(void)
{

}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | I r q 1 S e r v i c e |                             */
/*                      -----------------------                              */
/*    Called when Rx Frame sync word has been detected by ADF7021            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Irq1Service(void)
{
	ISR_ENTRY();
	AT91C_BASE_AIC->AIC_EOICR = 0;

//if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & TP5) == TP5 )
//	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, TP5 );
//else
//	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, TP5 );
	RxShiftCount = 0;;
	RxWordCount = 0;
	ISR_EXIT();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | F I Q I r q S e r v i c e |                         */
/*                      ---------------------------                          */
/*    Called when ADF7021 data clock transitions FIQ service                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void FIQIrqService(void)
{
	FSR_ENTRY();
	AT91C_BASE_AIC->AIC_EOICR = 0;

//	if( RXSTATE_SEARCH == RxState )	//if waiting for header sync
//	{
//	}
//	else if( RXSTATE_HDR == RxState )	//if reading in 660 header bits
//	{
//	}
//	else //reading voice data bits
	{
		RxShiftWord >>= 1;
		SyncTest >>= 1;
//		if(	  0x7AC8 ==  (SyncTest & 0x00007FFF) )		//stop
		if(	  0x0537 ==  (SyncTest & 0x00007FFF) )		//start
		{
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, TP5 );
		}

		if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & RXDATA_7021) )
		{
			RxShiftWord |= (1<<31);
			SyncTest |= (1<<31);
		}
		RxShiftCount++;
		if( RxShiftCount >= 32 )	//got 32 bits of data
		{	//put 32 bit word into next tmp frame location
			RxShiftCount = 0;
			TmpFrame.Buf32[RxWordCount++] = RxShiftWord;
			if(RxWordCount >= VDATAFRAME_SIZE)
			{	//got new frames worth of data so put in Rx frame Queue
				RxWordCount = 0;
				RFRxDataQueue[RFRxQHead].Buf32[0] = TmpFrame.Buf32[0]; 
				RFRxDataQueue[RFRxQHead].Buf32[1] = TmpFrame.Buf32[1]; 
				RFRxDataQueue[RFRxQHead++].Buf32[2] = TmpFrame.Buf32[2]; 
				RFRxQHead = RFRxQHead & (RFRQ_SIZE-1);
			}
		}
	}
AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, TP5 ) ;
	FSR_EXIT();
}
